@@ -20,7 +20,7 @@ static context_t ctx_localmem;
 static context_t ctx_mlocalmem;
 */
 #if defined(SYSV) || defined(WIN32) || defined(MMAP) || defined(HITACHI)
-#include "shmem.h"
+#include "armci_shmem.h"
 
 #if !defined(USE_SHMEM) && (defined(HITACHI) || defined(MULTI_CTX))
 #    define USE_SHMEM
@@ -312,7 +312,7 @@ void armci_shmem_memctl(armci_meminfo_t *meminfo) {
 
 /****** End: Non-collective memory allocation on shared memory systems *****/
 
-#ifdef MPI
+#ifdef MSG_COMMS_MPI
 /********************************************************************
  * Group Memory Allocation on shared memory systems for ARMCI Groups
 \*/
@@ -511,7 +511,7 @@ void armci_shmem_malloc_group(void *ptr_arr[], armci_size_t bytes,
     free(size_arr);
     ARMCI_PR_DBG("exit",0);
 }
-#endif /* ifdef MPI */
+#endif /* ifdef MSG_COMMS_MPI */
 
 #else
 
@@ -528,7 +528,7 @@ void* armci_shmem_memat(armci_meminfo_t *meminfo) {
 void armci_shmem_memctl(armci_meminfo_t *meminfo) {
   armci_die("armci_shmem_memctl should never be called on this system",0);  
 }
-# ifdef MPI
+# ifdef MSG_COMMS_MPI
   void armci_shmem_malloc_group(void *ptr_arr[], armci_size_t bytes,
                                 ARMCI_Group *group) {
       armci_die("armci_shmem_malloc_group should never be called on this system",0);
@@ -697,7 +697,7 @@ int ARMCI_Uses_shm()
     if(DEBUG_) fprintf(stderr,"%d:uses shmem %d\n",armci_me, uses);
     return uses;
 }
-#ifdef MPI
+#ifdef MSG_COMMS_MPI
 
 int ARMCI_Uses_shm_grp(ARMCI_Group *group) 
 {    
