@@ -20,10 +20,12 @@
 #include "message.h"
 #include "globalp.h"
 #include "armci.h"
+#include "ga_iterator.h"
 #include "ga-papi.h"
 #include "ga-wapi.h"
+#include "base.h"
 
-#ifdef MPI
+#ifdef MSG_COMMS_MPI
 extern ARMCI_Group* ga_get_armci_group_(int);
 #endif
 
@@ -82,6 +84,7 @@ void pnga_zero(Integer g_a)
 
       if (pnga_has_ghosts(g_a)) {
         pnga_zero_patch(g_a,lo,hi);
+        GA_POP_NAME;
         return;
       }
       pnga_access_ptr(g_a, lo, hi, &ptr, ld);
@@ -232,7 +235,43 @@ void *ptr_a, *ptr_b;
 }
 #endif
 
-
+void printBlock_x(char * banner, Integer type, void *ptr, Integer lo[],
+    Integer hi[], Integer ld[])
+{
+  Integer i,j;
+  Integer offset;
+  printf("p[%d] %s lo[0]: %d hi[0]: %d lo[1]: %d hi[1]: %d\n",
+      pnga_nodeid(),banner,lo[0],hi[0],lo[1],hi[1]);
+  printf("    ");
+  for (i=lo[0]; i<=hi[0]; i++) printf(" %12d",i);
+  printf("\n");
+  for (j=lo[1]; j<=hi[1]; j++) {
+    printf("p[%d] J: %d",pnga_nodeid(),j);
+    for (i=lo[0]; i<=hi[0]; i++) {
+      offset = (j-lo[1])*ld[0] + i-lo[0];
+      switch (type) {
+        case C_FLOAT:
+          printf(" %12.4f",*((float*)ptr+offset));
+          break;
+        case C_DBL:
+          printf(" %12.4f",*((double*)ptr+offset));
+          break;
+        case C_DCPL:
+          printf(" [%12.4f:%12.4f]",*((double*)ptr+2*offset),
+              *((double*)ptr+2*offset+1));
+          break;
+        case C_SCPL:
+          printf(" [%12.4f:%12.4f]",*((float*)ptr+2*offset),
+              *((float*)ptr+2*offset+1));
+          break;
+        default:
+          pnga_error("ga_matmul_basic: wrong data type", type);
+      }
+    }
+    printf("\n");
+  }
+  printf("\n\n");
+}
 
 /*\ COPY ONE GLOBAL ARRAY INTO ANOTHER
 \*/
@@ -246,8 +285,9 @@ Integer dimsb[MAXDIM],i;
 Integer a_grp, b_grp, anproc, bnproc;
 Integer num_blocks_a, num_blocks_b;
 Integer blocks[MAXDIM], block_dims[MAXDIM];
-void *ptr_a, *ptr_b;
+char *ptr_a, *ptr_b;
 int local_sync_begin,local_sync_end,use_put;
+_iterator_hdl hdl;
 
    GA_PUSH_NAME("ga_copy");
 
@@ -296,6 +336,7 @@ int local_sync_begin,local_sync_end,use_put;
         Copy operation is straightforward */
 
      if (use_put) {
+#if 0
        if (num_blocks_a < 0) {
          pnga_distribution(g_a, me_a, lo, hi);
          if(lo[0]>0){
@@ -342,7 +383,14 @@ int local_sync_begin,local_sync_end,use_put;
            }
          }
        }
+#else
+       pnga_local_iterator_init(g_a, &hdl);
+       while (pnga_local_iterator_next(&hdl,lo,hi,&ptr_a,ld)) {
+         pnga_put(g_b, lo, hi, ptr_a, ld);
+       }
+#endif
      } else {
+#if 0
        if (num_blocks_b < 0) {
          pnga_distribution(g_b, me_b, lo, hi);
          if(lo[0]>0){
@@ -389,6 +437,12 @@ int local_sync_begin,local_sync_end,use_put;
            }
          }
        }
+#else
+       pnga_local_iterator_init(g_b, &hdl);
+       while (pnga_local_iterator_next(&hdl,lo,hi,&ptr_b,ld)) {
+         pnga_get(g_a, lo, hi, ptr_b, ld);
+       }
+#endif
      }
    } else {
      /* One global array is mirrored and the other is not */
@@ -621,7 +675,7 @@ Integer bndim, bdims[MAXDIM];
    } else {
      if (a_grp == -1) {
        armci_msg_gop_scope(SCOPE_ALL,value,alen,"+",atype);
-#ifdef MPI
+#ifdef MSG_COMMS_MPI
      } else {
        armci_msg_group_gop_scope(SCOPE_ALL,value,alen,"+",atype,
            ga_get_armci_group_((int)a_grp));
@@ -996,10 +1050,11 @@ void pnga_transpose(Integer g_a, Integer g_b)
 Integer me = pnga_nodeid();
 Integer nproc = pnga_nnodes(); 
 Integer atype, btype, andim, adims[MAXDIM], bndim, bdims[MAXDIM];
-Integer lo[2],hi[2];
+Integer lo[2],hi[2],ld[2];
 int local_sync_begin,local_sync_end;
 Integer num_blocks_a;
 char *ptr_tmp, *ptr_a;
+_iterator_hdl hdl;
 
     GA_PUSH_NAME("ga_transpose");
     
@@ -1015,6 +1070,32 @@ char *ptr_tmp, *ptr_a;
     if(bndim != 2 || andim != 2) pnga_error("dimension must be 2",0);
     if(atype != btype ) pnga_error("array type mismatch ", 0L);
 
+#if 1
+    pnga_local_iterator_init(g_a, &hdl);
+    while (pnga_local_iterator_next(&hdl,lo,hi,&ptr_a,ld)) {
+      Integer idx;
+      Integer block_dims[MAXDIM];
+      Integer nelem, lob[2], hib[2], nrow, ncol;
+      int i, size=GAsizeofM(atype);
+
+      nelem = (hi[0]-lo[0]+1)*(hi[1]-lo[1]+1);
+      ptr_tmp = (char *) ga_malloc(nelem, atype, "transpose_tmp");
+
+      nrow   = hi[0] -lo[0]+1;
+      ncol   = hi[1] -lo[1]+1; 
+      nelem  = nrow*ncol;
+      lob[0] = lo[1]; lob[1] = lo[0];
+      hib[0] = hi[1]; hib[1] = hi[0];
+      for(i = 0; i < ncol; i++){
+        char *ptr = ptr_tmp + i*size;
+
+        snga_local_transpose(atype, ptr_a, nrow, ncol*size, ptr);
+        ptr_a += ld[0]*size;
+      }
+      pnga_put(g_b, lob, hib, ptr_tmp ,&ncol);
+      ga_free(ptr_tmp);
+    }
+#else
     num_blocks_a = pnga_total_blocks(g_a);
 
     if (num_blocks_a < 0) {
@@ -1138,7 +1219,7 @@ char *ptr_tmp, *ptr_a;
       }
       ga_free(ptr_tmp);
     }
-
+#endif
     if(local_sync_end)pnga_sync();
     GA_POP_NAME;
 }
